function [ J ] = FIM_AoA_2D( A, N, N_L, sigma_AOA, nodes, d )

%%      Author: Panos N. Alevizos - July 2016
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Calculation of Fisher information matrix for 2D-AoA measurements
%
%
%   Inputs:
%   A : N+L x N+L connectivity matrix (caution: it may be assymetric)
%   N : Number of agents
%   N_L : Number of all nodes in the network (agents+anchors)
%   sigma_AOA : the standard deviation for AoA measurements
%   nodes : all node position in the 2D plane
%   d : N+L x N+L distance matrix
%
%
%   Output:
%   J : Fisher information matrix associated with 2D AoA measurements
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

J = zeros(2*N,2*N);
P2 = [0 -1 ; 1 0];
for i = 1 : N
    for j = 1 : N
        if(i==j)
            for k=1:N_L
                if(A(i,k) == 1 )
                    grad_v_nj = (1/(d(i,k)^2))*P2*( nodes(:,i) - nodes(:,k) );
                    J(2*i-1:2*i,2*j-1:2*j) = J(2*i-1:2*i,2*j-1:2*j) + ( ((1/sigma_AOA^2) * (grad_v_nj) * (grad_v_nj)' ));
                end
            end
            for k=1:N_L
                if(A(k,i) == 1 )
                    grad_v_nj = (1/(d(i,k)^2))*P2*( nodes(:,k) - nodes(:,i) );
                    J(2*i-1:2*i,2*j-1:2*j) = J(2*i-1:2*i,2*j-1:2*j) + ( ((1/sigma_AOA^2) * (grad_v_nj) * (grad_v_nj)' ));
                end
            end
        else
            if(A(i,j) == 1 )
                grad_v_nj = (1/(d(i,j)^2))*P2*( nodes(:,i) - nodes(:,j) );
                J(2*i-1:2*i,2*j-1:2*j) = J(2*i-1:2*i,2*j-1:2*j) -( (1/sigma_AOA^2) * (grad_v_nj) * (grad_v_nj)' ) ;
                
            end
            if(A(j,i) == 1 )
                grad_v_nj = (1/(d(i,j)^2))*P2*( nodes(:,j) - nodes(:,i) );
                J(2*i-1:2*i,2*j-1:2*j) = J(2*i-1:2*i,2*j-1:2*j) -( (1/sigma_AOA^2) * (grad_v_nj) * (grad_v_nj)' ) ;
                
            end
        end
    end
end


end

