function [ J ] = FIM_RSS_2D( A, N, N_L, sigma_RSS, nu, nodes, d )

%%      Author: Panos N. Alevizos - July 2016
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Calculation of Fisher information matrix for 2D-RSS measurements 
%
%
%   Inputs:
%   A : N+L x N+L connectivity matrix (caution: it may be assymetric)
%   N : Number of agents
%   N_L : Number of all nodes in the network (agents+anchors)
%   sigma_RSS : the standard deviation fo RSS measurements
%   nodes : all node position in the 2D plane
%   d : N+L x N+L distance matrix
%
%
%   Output:
%   J : Fisher information matrix associated with 2D RSS measurements
%   without any assumption regarding the antennas
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

J = zeros(2*N,2*N);
for i = 1 : N
    for j = 1 : N
        if(i==j)
            for k=1:N_L
                if(A(i,k) == 1 )
                    J(2*i-1:2*i,2*j-1:2*j) = J(2*i-1:2*i,2*j-1:2*j) + (  ( ( (100*nu^2)/( (log(10))^2 * sigma_RSS^2 )) *...
                        (( nodes(:,i) - nodes(:,k) )*( nodes(:,i) - nodes(:,k) )' ) /  (d(i,k)^4)));
                end
                if(A(k,i) == 1 )
                    J(2*i-1:2*i,2*j-1:2*j) = J(2*i-1:2*i,2*j-1:2*j) + (  ( ( (100*nu^2)/( (log(10))^2 * sigma_RSS^2 )) *...
                        (( nodes(:,k) - nodes(:,i) )*( nodes(:,k) - nodes(:,i) )' ) /  (d(i,k)^4)));
                end
            end
        else
            if(A(i,j) == 1 )
                
                J(2*i-1:2*i,2*j-1:2*j) = J(2*i-1:2*i,2*j-1:2*j) - ( ( (100*nu^2)/( (log(10))^2 * sigma_RSS^2 )) *...
                    (( nodes(:,i) - nodes(:,j) )*( nodes(:,i) - nodes(:,j) )' ) /  (d(i,j)^4)) ;
                
            end
            if(A(j,i) == 1 )
                
                J(2*i-1:2*i,2*j-1:2*j) = J(2*i-1:2*i,2*j-1:2*j) - ( ( (100*nu^2)/( (log(10))^2 * sigma_RSS^2 )) *...
                    (( nodes(:,j) - nodes(:,i) )*( nodes(:,j) - nodes(:,i) )' ) /  (d(i,j)^4)) ;
                
            end
        end
    end
end


end

