function [ J ] = FIM_ToA_2D( A, N, N_L, sigma_TOA, prop_velocity, nodes, d )

%%      Author: Panos N. Alevizos - July 2016
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Calculation of Fisher information matrix for 2D-ToA measurements 
%
%
%   Inputs:
%   A : N+L x N+L connectivity matrix (caution: it may be assymetric)
%   N : Number of agents
%   N_L : Number of all nodes in the network (agents+anchors)
%   sigma_TOA : the standard deviation fo ToA measurements
%   prop_velocity : propagation velocity = 3 x 10^8
%   nodes : all node position in the 2D plane
%   d : N+L x N+L distance matrix
%
%
%   Output:
%   J : Fisher information matrix associated with 2D ToA measurements
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


J = zeros(2*N,2*N);
for i = 1 : N
    for j = 1 : N
        if(i==j) 
            for k=1:N_L
                if(A(i,k) == 1 )
                    J(2*i-1:2*i,2*j-1:2*j) = J(2*i-1:2*i,2*j-1:2*j) + (((1/(prop_velocity^2 * sigma_TOA^2)) *...
                        (( nodes(:,i) - nodes(:,k) )*( nodes(:,i) - nodes(:,k) )' ) /  (d(i,k)^2)));
                end
            end
            for k=1:N_L
                if(A(k,i) == 1 )
                    J(2*i-1:2*i,2*j-1:2*j) = J(2*i-1:2*i,2*j-1:2*j) + (((1/(prop_velocity^2 * sigma_TOA^2)) *...
                        (( nodes(:,k) - nodes(:,i) )*( nodes(:,k) - nodes(:,i) )' ) /  (d(i,k)^2)));
                end
            end
        else
            if(A(i,j) == 1 )
                J(2*i-1:2*i,2*j-1:2*j) = J(2*i-1:2*i,2*j-1:2*j) - ((1/(prop_velocity^2 * sigma_TOA^2)) *...
                    (( nodes(:,i) - nodes(:,j) )*( nodes(:,i) - nodes(:,j) )' ) /  (d(i,j)^2));
                
            end
            if(A(j,i) == 1 )
                J(2*i-1:2*i,2*j-1:2*j) = J(2*i-1:2*i,2*j-1:2*j) - ((1/(prop_velocity^2 * sigma_TOA^2)) *...
                    (( nodes(:,j) - nodes(:,i) )*( nodes(:,j) - nodes(:,i) )' ) /  (d(i,j)^2));
                
            end
        end
    end
end


end

