function [ J ] = FIM_RSS_dipoles_3D( A, N, N_L, sigma_RSS, nu, nodes, d )


%%      Author: Panos N. Alevizos - July 2016
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Calculation of Fisher information matrix for 3D-RSS measurements with
%dipole antennas
%
%
%   Inputs:
%   A : N+L x N+L connectivity matrix (caution: it may be assymetric)
%   N : Number of agents
%   N_L : Number of all nodes in the network (agents+anchors)
%   sigma_RSS : the standard deviation fo RSS measurements
%   nu : PLE
%   nodes : all node position in the 3D plane
%   d : N+L x N+L distance matrix
%
%
%   Output:
%   J : Fisher information matrix associated with 3D-RSS measurements with
%   dipole antennas
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

J = zeros(3*N,3*N);
for i = 1 : N
    for j = 1 : N
        if(i==j)
            for k=1:N_L
                if(A(i,k) == 1 )
                    grad_v_nj = ((60/log(10))/(d(i,k)^2))*...
                        [((nodes(1,i) - nodes(1,k))*(nodes(3,i) - nodes(3,k))^2)...
                        / (d(i,k)^2 - (nodes(3,i) - nodes(3,k))^2 ) ;...
                        ((nodes(2,i) - nodes(2,k))*(nodes(3,i) - nodes(3,k))^2)...
                        / (d(i,k)^2 - (nodes(3,i) - nodes(3,k))^2 ) ;...
                        -(nodes(3,i) - nodes(3,k)) ];
                    grad_g_nj = -((10*nu)/(log(10)) )* ((nodes(:,i) - nodes(:,k))/d(i,k)^2);
                    J(3*i-2 : 3*i , 3*j-2 : 3*j) = J(3*i-2 : 3*i , 3*j-2 : 3*j) + (  ( 1 /( sigma_RSS^2 ) ) *...
                        (grad_g_nj+ grad_v_nj)*(grad_g_nj+ grad_v_nj)' );
                end
            end
            for k=1:N_L
                if(A(k,i) == 1 )
                    grad_v_nj = ((60/log(10))/(d(i,k)^2))*...
                        [((nodes(1,k) - nodes(1,i))*(nodes(3,k) - nodes(3,i))^2)...
                        / (d(i,k)^2 - (nodes(3,k) - nodes(3,i))^2 ) ;...
                        ((nodes(2,k) - nodes(2,i))*(nodes(3,k) - nodes(3,i))^2)...
                        / (d(i,k)^2 - (nodes(3,k) - nodes(3,i))^2 ) ;...
                        -(nodes(3,k) - nodes(3,i)) ];
                    grad_g_nj = -((10*nu)/(log(10)) )* ((nodes(:,k) - nodes(:,i))/d(i,k)^2);
                    J(3*i-2 : 3*i , 3*j-2 : 3*j) = J(3*i-2 : 3*i , 3*j-2 : 3*j) + (  ( 1 /( sigma_RSS^2 ) ) *...
                        (grad_g_nj+ grad_v_nj)*(grad_g_nj+ grad_v_nj)' );
                end
            end
        else
            if(A(i,j) == 1 )
                grad_v_nj = ((60/log(10))/(d(i,j)^2))*...
                    [((nodes(1,i) - nodes(1,j))*(nodes(3,i) - nodes(3,j))^2)...
                    / (d(i,j)^2 - (nodes(3,i) - nodes(3,j))^2 ) ;...
                    ((nodes(2,i) - nodes(2,j))*(nodes(3,i) - nodes(3,j))^2)...
                    / (d(i,j)^2 - (nodes(3,i) - nodes(3,j))^2 ) ;...
                    -(nodes(3,i) - nodes(3,j)) ];
                grad_g_nj = -((10*nu)/(log(10)) )* ((nodes(:,i) - nodes(:,j))/d(i,j)^2);
                
                J(3*i-2 : 3*i , 3*j-2 : 3*j) = J(3*i-2 : 3*i , 3*j-2 : 3*j) - (( 1 /( sigma_RSS^2 ) ) *...
                        (grad_g_nj+ grad_v_nj)*(grad_g_nj+ grad_v_nj)' ) ;
                
            end
            if(A(j,i) == 1 )
                grad_v_nj = ((60/log(10))/(d(i,j)^2))*...
                    [((nodes(1,j) - nodes(1,i))*(nodes(3,j) - nodes(3,i))^2)...
                    / (d(i,j)^2 - (nodes(3,j) - nodes(3,i))^2 ) ;...
                    ((nodes(2,j) - nodes(2,i))*(nodes(3,j) - nodes(3,i))^2)...
                    / (d(i,j)^2 - (nodes(3,j) - nodes(3,i))^2 ) ;...
                    -(nodes(3,j) - nodes(3,i)) ];
                grad_g_nj = -((10*nu)/(log(10)) )* ((nodes(:,j) - nodes(:,i))/d(i,j)^2);
                
                J(3*i-2 : 3*i , 3*j-2 : 3*j) = J(3*i-2 : 3*i , 3*j-2 : 3*j) - (( 1 /( sigma_RSS^2 ) ) *...
                        (grad_g_nj+ grad_v_nj)*(grad_g_nj+ grad_v_nj)' ) ;
                
            end
        end
    end
end


end

